@@ -11,8 +11,6 @@ import (
 	"errors"
 	"fmt"
 	"net/url"
-	"os"
-	"path/filepath"
 	"sort"
 	"strings"
 	"time"
@@ -55,22 +53,22 @@ var (
 	// "-X 'ariga.io/atlas/cmd/atlas/internal/cmdapi.version=v0.1.2'"
 	version string
 
-	// schemaCmd represents the subcommand 'atlas version'.
+	// versionCmd represents the subcommand 'atlas version'.
 	versionCmd = &cobra.Command{
 		Use:   "version",
 		Short: "Prints this Atlas CLI version information.",
 		Run: func(cmd *cobra.Command, _ []string) {
 			var (
-				f    = "atlas "
+				f    = versionFmt
 				args []any
 			)
 			if flavor != "" {
 				f += "%s "
 				args = append(args, flavor)
 			}
-			f += "version %s\n%s\n"
+			f += "version %s\n%s\n%s"
 			v, u := parseV(version)
-			args = append(args, v, u)
+			args = append(args, v, u, versionInfo)
 			cmd.Printf(f, args...)
 		},
 	}
@@ -540,7 +538,7 @@ func stateReader(ctx context.Context, env *Env, config *stateReaderConfig) (*cmd
 	switch scheme {
 	// "file" scheme is valid for both migration directory and HCL paths.
 	case cmdext.SchemaTypeFile:
-		switch ext, err := filesExt(excfg.URLs); {
+		switch ext, err := cmdext.FilesExt(excfg.URLs); {
 		case err != nil:
 			return nil, err
 		case ext == cmdext.FileTypeHCL:
@@ -601,54 +599,3 @@ func stateReader(ctx context.Context, env *Env, config *stateReaderConfig) (*cmd
 		}, nil
 	}
 }
-
-func filesExt(urls []*url.URL) (string, error) {
-	var path, ext string
-	set := func(curr string) error {
-		switch e := filepath.Ext(curr); {
-		case e != cmdext.FileTypeHCL && e != cmdext.FileTypeSQL:
-			return fmt.Errorf("unknown schema file: %q", curr)
-		case ext != "" && ext != e:
-			return fmt.Errorf("ambiguous schema: both SQL and HCL files found: %q, %q", path, curr)
-		default:
-			path, ext = curr, e
-			return nil
-		}
-	}
-	for _, u := range urls {
-		path := filepath.Join(u.Host, u.Path)
-		switch fi, err := os.Stat(path); {
-		case err != nil:
-			return "", err
-		case fi.IsDir():
-			files, err := os.ReadDir(path)
-			if err != nil {
-				return "", err
-			}
-			for _, f := range files {
-				switch filepath.Ext(f.Name()) {
-				// Ignore unknown extensions in case we read directories.
-				case cmdext.FileTypeHCL, cmdext.FileTypeSQL:
-					if err := set(f.Name()); err != nil {
-						return "", err
-					}
-				}
-			}
-		default:
-			if err := set(fi.Name()); err != nil {
-				return "", err
-			}
-		}
-	}
-	switch {
-	case ext != "":
-	case len(urls) == 1 && (urls[0].Host != "" || urls[0].Path != ""):
-		return "", fmt.Errorf(
-			"%q contains neither SQL nor HCL files",
-			filepath.Base(filepath.Join(urls[0].Host, urls[0].Path)),
-		)
-	default:
-		return "", errors.New("schema contains neither SQL nor HCL files")
-	}
-	return ext, nil
-}
