@@ -13,10 +13,12 @@ import (
 	"text/template"
 
 	"ariga.io/atlas/cmd/atlas/internal/cloudapi"
+	"ariga.io/atlas/cmd/atlas/internal/cmdext"
 	cmdmigrate "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/cmd/atlas/internal/migratelint"
 	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/migrate"
+	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlcheck"
 	"ariga.io/atlas/sql/sqlclient"
 
@@ -48,6 +50,14 @@ func init() {
 	Root.AddCommand(migrateCmd)
 }
 
+// Project represents an atlas.hcl project config file.
+type Project struct {
+	Envs  []*Env `spec:"env"`  // List of environments
+	Lint  *Lint  `spec:"lint"` // Optional global lint policy
+	Diff  *Diff  `spec:"diff"` // Optional global diff policy
+	cloud *cmdext.AtlasConfig
+}
+
 // migrateLintSetFlags allows setting extra flags for the 'migrate lint' command.
 func migrateLintSetFlags(*cobra.Command, *migrateLintFlags) {}
 
@@ -135,3 +145,13 @@ func setEnvs(context.Context, []*Env) {}
 
 // specOptions are the options for the schema spec.
 var specOptions []schemahcl.Option
+
+// diffOptions returns environment-aware diff options.
+func diffOptions(_ *cobra.Command, env *Env) []schema.DiffOption {
+	return env.DiffOptions()
+}
+
+// openClient allows opening environment-aware clients.
+func (*Env) openClient(ctx context.Context, u string) (*sqlclient.Client, error) {
+	return sqlclient.Open(ctx, u)
+}
