@@ -15,6 +15,7 @@ import (
 	"net/url"
 	"os"
 	"os/exec"
+	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
@@ -104,10 +105,16 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 			RunE: RunE(func(cmd *cobra.Command, args []string) (cmdErr error) {
 				switch {
 				case GlobalFlags.SelectedEnv == "":
-					if err := migrateFlagsFromConfig(cmd); err != nil {
+					// Env not selected, but the
+					// -c flag might be set.
+					env, err := selectEnv(cmd)
+					if err != nil {
 						return err
 					}
-					return migrateApplyRun(cmd, args, flags, &MigrateReport{}) // nop reporter
+					if err := setMigrateEnvFlags(cmd, env); err != nil {
+						return err
+					}
+					return migrateApplyRun(cmd, args, flags, env, &MigrateReport{}) // nop reporter
 				default:
 					project, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, GlobalFlags.Vars)
 					if err != nil {
@@ -125,10 +132,10 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 					}()
 					return cmdEnvsRun(envs, setMigrateEnvFlags, cmd, func(env *Env) error {
 						// Report deployments only if one of the migration directories is a cloud directory.
-						if u, err := url.Parse(flags.dirURL); err == nil && u.Scheme != cmdmigrate.DirTypeFile {
+						if u, err := url.Parse(flags.dirURL); err == nil && u.Scheme == cmdmigrate.DirTypeAtlas {
 							hasRemote = true
 						}
-						return migrateApplyRun(cmd, args, flags, set.ReportFor(flags, env))
+						return migrateApplyRun(cmd, args, flags, env, set.ReportFor(flags, env))
 					})
 				}
 			}),
@@ -152,8 +159,8 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 	return cmd
 }
 
-// migrateApplyCmd represents the 'atlas migrate apply' subcommand.
-func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags, mr *MigrateReport) (err error) {
+// migrateApplyRun represents the 'atlas migrate apply' subcommand.
+func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags, env *Env, mr *MigrateReport) (err error) {
 	var (
 		count int
 		ctx   = cmd.Context()
@@ -179,7 +186,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags,
 	if flags.url == "" {
 		return errors.New(`required flag "url" not set`)
 	}
-	client, err := sqlclient.Open(ctx, flags.url)
+	client, err := env.openClient(ctx, flags.url)
 	if err != nil {
 		return err
 	}
@@ -304,8 +311,8 @@ type (
 // NewReportProvider returns a new ReporterProvider.
 func NewReportProvider(ctx context.Context, p *Project, envs []*Env, flags *migrateApplyFlags) (*MigrateReportSet, error) {
 	c := cloudapi.FromContext(ctx)
-	if p.cfg.Client != nil {
-		c = p.cfg.Client
+	if p.cloud.Client != nil {
+		c = p.cloud.Client
 	}
 	s := &MigrateReportSet{
 		client: c,
@@ -376,6 +383,7 @@ func (s *MigrateReportSet) ReportFor(flags migrateApplyFlags, e *Env) *MigrateRe
 		env: e,
 		done: func(r *cloudapi.ReportMigrationInput) {
 			s.done++
+			r.DryRun = flags.dryRun
 			s.Log[len(s.Log)-1].EndTime = time.Now()
 			if r.Error != nil && *r.Error != "" {
 				s.StepLogError(*r.Error)
@@ -480,10 +488,10 @@ func (r *MigrateReport) Done(cmd *cobra.Command, flags migrateApplyFlags) error
 		dirName = cloudapi.DefaultDirName
 	// Directory slug.
 	default:
-		dirName = filepath.Join(u.Host, u.Path)
+		dirName = path.Join(u.Host, u.Path)
 	}
 	r.done(&cloudapi.ReportMigrationInput{
-		ProjectName:  r.env.cfg.Project,
+		ProjectName:  r.env.config.cloud.Project,
 		EnvName:      r.env.Name,
 		DirName:      dirName,
 		AtlasVersion: operatorVersion(),
@@ -543,7 +551,12 @@ func (r *MigrateReport) Done(cmd *cobra.Command, flags migrateApplyFlags) error
 
 // CloudEnabled reports if cloud reporting is enabled.
 func (r *MigrateReport) CloudEnabled(ctx context.Context) bool {
-	return r.env != nil && r.env.cfg != nil && r.env.cfg.Project != "" && (r.env.cfg.Client != nil || cloudapi.FromContext(ctx) != nil)
+	if r.env == nil || r.env.cloud == nil {
+		return false // The --env was not set.
+	}
+	cloud := r.env.cloud
+	// Cloud reporting is enabled only if there is a cloud connection.
+	return cloud.Project != "" && (cloud.Client != nil || cloudapi.FromContext(ctx) != nil)
 }
 
 func logApply(cmd *cobra.Command, w io.Writer, flags migrateApplyFlags, r *cmdlog.MigrateApply) error {
@@ -677,7 +690,7 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags, e
 			Indent:  indent,
 			Dir:     dir,
 			Dev:     dev,
-			Options: env.DiffOptions(),
+			Options: diffOptions(cmd, env),
 		})
 		return maskNoPlan(cmd, err)
 	}
@@ -696,7 +709,7 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags, e
 	opts := []migrate.PlannerOption{
 		migrate.PlanFormat(f),
 		migrate.PlanWithIndent(indent),
-		migrate.PlanWithDiffOptions(env.DiffOptions()...),
+		migrate.PlanWithDiffOptions(diffOptions(cmd, env)...),
 	}
 	if dev.URL.Schema != "" {
 		// Disable tables qualifier in schema-mode.
@@ -943,7 +956,7 @@ type migrateLintFlags struct {
 	latest            uint   // --latest 1
 	gitBase, gitDir   string // --git-base master --git-dir /path/to/git/repo
 	// Not enabled by default.
-	dirBase string // --base atlas://graph
+	dirBase string // --base atlas://myapp
 	web     bool   // Open the web browser
 	context string // Run context. See cloudapi.ContextInput.
 }
@@ -1161,7 +1174,7 @@ func migrateSetRun(cmd *cobra.Command, args []string, flags migrateSetFlags) (re
 	default:
 		return fmt.Errorf("accepts 1 arg(s), received %d", n)
 	}
-	log := &cmdlog.MigrateSet{Context: ctx}
+	log := cmdlog.NewMigrateSet(ctx)
 	for _, r := range revs {
 		// Check all existing revisions and ensure they precede the given version. If we encounter a partially
 		// applied revision, or one with errors, mark them "fixed".
@@ -1613,7 +1626,7 @@ func printChecksumError(cmd *cobra.Command, err error) {
 	out := cmd.OutOrStderr()
 	fmt.Fprintln(out, "You have a checksum error in your migration directory.")
 	if csErr := (&migrate.ChecksumError{}); errors.As(err, &csErr) {
-		fmt.Fprintf(out, "\n\tL %d: %s was %s\n\n", csErr.Line, csErr.File, csErr.Reason)
+		fmt.Fprintf(out, "\n\tL%d: %s was %s\n\n", csErr.Line, csErr.File, csErr.Reason)
 	}
 	fmt.Fprintf(
 		out,
@@ -1693,6 +1706,13 @@ func setMigrateEnvFlags(cmd *cobra.Command, env *Env) error {
 		if err := maySetFlag(cmd, flagExecOrder, strings.ReplaceAll(strings.ToLower(env.Migration.ExecOrder), "_", "-")); err != nil {
 			return err
 		}
+	case "down":
+		if err := maySetFlag(cmd, flagFormat, env.Format.Migrate.Down); err != nil {
+			return err
+		}
+		if err := maySetFlag(cmd, flagLockTimeout, env.Migration.LockTimeout); err != nil {
+			return err
+		}
 	case "diff", "checkpoint":
 		if err := maySetFlag(cmd, flagLockTimeout, env.Migration.LockTimeout); err != nil {
 			return err
@@ -1798,23 +1818,23 @@ func (d *editDir) WriteFile(name string, b []byte) (err error) {
 
 // edit allows editing the file content using editor.
 func edit(name string, src []byte) ([]byte, error) {
-	path := filepath.Join(os.TempDir(), name)
-	if err := os.WriteFile(path, src, 0644); err != nil {
+	p := filepath.Join(os.TempDir(), name)
+	if err := os.WriteFile(p, src, 0644); err != nil {
 		return nil, fmt.Errorf("write source content to temp file: %w", err)
 	}
-	defer os.Remove(path)
+	defer os.Remove(p)
 	editor := "vi"
 	if e := os.Getenv("EDITOR"); e != "" {
 		editor = e
 	}
-	cmd := exec.Command("sh", "-c", editor+" "+path)
+	cmd := exec.Command("sh", "-c", editor+" "+p)
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
 	if err := cmd.Run(); err != nil {
 		return nil, fmt.Errorf("exec edit: %w", err)
 	}
-	b, err := os.ReadFile(path)
+	b, err := os.ReadFile(p)
 	if err != nil {
 		return nil, fmt.Errorf("read edited temp file: %w", err)
 	}
@@ -1829,11 +1849,6 @@ type (
 	dryRunRevisions struct{ migrate.RevisionReadWriter }
 )
 
-// QueryContext overrides the wrapped schema.ExecQuerier to not execute any SQL.
-func (dryRunDriver) QueryContext(context.Context, string, ...any) (*sql.Rows, error) {
-	return nil, nil
-}
-
 // ExecContext overrides the wrapped schema.ExecQuerier to not execute any SQL.
 func (dryRunDriver) ExecContext(context.Context, string, ...any) (sql.Result, error) {
 	return nil, nil
