@@ -5,6 +5,7 @@
 package cmdapi
 
 import (
+	"context"
 	"fmt"
 	"net/url"
 	"os"
@@ -28,14 +29,6 @@ import (
 )
 
 type (
-	// Project represents an atlas.hcl project config file.
-	Project struct {
-		Envs []*Env `spec:"env"`  // List of environments
-		Lint *Lint  `spec:"lint"` // Optional global lint policy
-		Diff *Diff  `spec:"diff"` // Optional global diff policy
-		cfg  *cmdext.AtlasConfig
-	}
-
 	// Env represents an Atlas environment.
 	Env struct {
 		// Name for this environment.
@@ -68,7 +61,8 @@ type (
 		Format Format `spec:"format"`
 
 		schemahcl.DefaultExtension
-		cfg *cmdext.AtlasConfig
+		cloud  *cmdext.AtlasConfig
+		config *Project
 	}
 
 	// Migration represents the migration directory for the Env.
@@ -146,6 +140,8 @@ type (
 		Migrate struct {
 			// Apply configures the formatting for 'migrate apply'.
 			Apply string `spec:"apply"`
+			// Down configures the formatting for 'migrate down'.
+			Down string `spec:"down"`
 			// Lint configures the formatting for 'migrate lint'.
 			Lint string `spec:"lint"`
 			// Status configures the formatting for 'migrate status'.
@@ -154,6 +150,8 @@ type (
 			Diff string `spec:"diff"`
 		} `spec:"migrate"`
 		Schema struct {
+			// Inspect configures the formatting for 'schema inspect'.
+			Inspect string `spec:"inspect"`
 			// Apply configures the formatting for 'schema apply'.
 			Apply string `spec:"apply"`
 			// Apply configures the formatting for 'schema diff'.
@@ -390,7 +388,7 @@ func EnvByName(cmd *cobra.Command, name string, vars map[string]cty.Value) (*Pro
 		setEnvs(cmd.Context(), envs[name])
 	}()
 	if p, e, ok := envsCache.load(GlobalFlags.ConfigURL, name, vars); ok {
-		return p, e, nil
+		return p, e, maySetTokenContext(cmd, p)
 	}
 	u, err := url.Parse(GlobalFlags.ConfigURL)
 	if err != nil {
@@ -406,18 +404,14 @@ func EnvByName(cmd *cobra.Command, name string, vars map[string]cty.Value) (*Pro
 		}
 		return nil, nil, err
 	}
-	project, err := parseConfig(path, name, vars)
+	project, err := parseConfig(cmd.Context(), path, name, vars)
 	if err != nil {
 		return nil, nil, err
 	}
 	// The project token predates 'atlas login' command. If exists,
 	// attach it to the context to indicate the user is authenticated.
-	if project.cfg.Token != "" && project.cfg.Client != nil {
-		ctx, err := withTokenContext(cmd.Context(), project.cfg.Token, project.cfg.Client)
-		if err != nil {
-			return nil, nil, err
-		}
-		cmd.SetContext(ctx)
+	if err := maySetTokenContext(cmd, project); err != nil {
+		return nil, nil, err
 	}
 	if err := project.Lint.remainedLog(); err != nil {
 		return nil, nil, err
@@ -449,12 +443,25 @@ func EnvByName(cmd *cobra.Command, name string, vars map[string]cty.Value) (*Pro
 		// return only the project.
 		return project, nil, nil
 	case len(envs[name]) == 0:
-		return nil, nil, fmt.Errorf("env %q not defined in project file", name)
+		return nil, nil, fmt.Errorf("env %q not defined in config file", name)
 	default:
 		return project, envs[name], nil
 	}
 }
 
+// maySetTokenContext sets the token context and cloud client if
+// defined in the project file.
+func maySetTokenContext(cmd *cobra.Command, p *Project) error {
+	if p.cloud.Token != "" && p.cloud.Client != nil {
+		ctx, err := withTokenContext(cmd.Context(), p.cloud.Token, p.cloud.Client)
+		if err != nil {
+			return err
+		}
+		cmd.SetContext(ctx)
+	}
+	return nil
+}
+
 type (
 	envCacheK struct {
 		path, env, vars string
@@ -490,7 +497,7 @@ const (
 	defaultConfigPath = "file://atlas.hcl"
 )
 
-func parseConfig(path, env string, vars map[string]cty.Value) (*Project, error) {
+func parseConfig(ctx context.Context, path, env string, vars map[string]cty.Value) (*Project, error) {
 	pr, err := partialParse(path, env)
 	if err != nil {
 		return nil, err
@@ -499,13 +506,14 @@ func parseConfig(path, env string, vars map[string]cty.Value) (*Project, error)
 	if err != nil {
 		return nil, err
 	}
-	cfg := &cmdext.AtlasConfig{
+	cloud := &cmdext.AtlasConfig{
 		Project: cloudapi.DefaultProjectName,
 	}
 	state := schemahcl.New(
 		append(
 			append(cmdext.SpecOptions, specOptions...),
-			cfg.InitBlock(),
+			cloud.InitBlock(),
+			schemahcl.WithContext(ctx),
 			schemahcl.WithScopedEnums("env.migration.format", cmdmigrate.Formats...),
 			schemahcl.WithScopedEnums("env.migration.exec_order", "LINEAR", "LINEAR_SKIP", "NON_LINEAR"),
 			schemahcl.WithScopedEnums("env.lint.review", ReviewModes...),
@@ -532,12 +540,12 @@ func parseConfig(path, env string, vars map[string]cty.Value) (*Project, error)
 			}),
 		)...,
 	)
-	p := &Project{Lint: &Lint{}, Diff: &Diff{}, cfg: cfg}
+	p := &Project{Lint: &Lint{}, Diff: &Diff{}, cloud: cloud}
 	if err := state.Eval(pr, p, vars); err != nil {
 		return nil, err
 	}
 	for _, e := range p.Envs {
-		e.cfg = cfg
+		e.config, e.cloud = p, cloud
 	}
 	return p, nil
 }
@@ -553,7 +561,7 @@ func partialParse(path, env string) (*hclparse.Parser, error) {
 	if err != nil {
 		return nil, err
 	}
-	var used []*hclsyntax.Block
+	var labeled, nonlabeled, used []*hclsyntax.Block
 	for _, b := range fi.Body.(*hclsyntax.Body).Blocks {
 		switch b.Type {
 		case blockEnv:
@@ -562,18 +570,26 @@ func partialParse(path, env string) (*hclparse.Parser, error) {
 			case env == "" && n == 0:
 			// Exact env was selected.
 			case n == 1 && b.Labels[0] == env:
-				used = append(used, b)
+				labeled = append(labeled, b)
 			// Dynamic env selection.
 			case n == 0 && b.Body != nil && b.Body.Attributes[schemahcl.AttrName] != nil:
 				t, ok := b.Body.Attributes[schemahcl.AttrName].Expr.(*hclsyntax.ScopeTraversalExpr)
 				if ok && len(t.Traversal) == 2 && t.Traversal.RootName() == refAtlas && t.Traversal[1].(hcl.TraverseAttr).Name == blockEnv {
-					used = append(used, b)
+					nonlabeled = append(nonlabeled, b)
 				}
 			}
 		default:
 			used = append(used, b)
 		}
 	}
+	// Labeled blocks take precedence
+	// over non-labeled env blocks.
+	switch {
+	case len(labeled) > 0:
+		used = append(used, labeled...)
+	case len(nonlabeled) > 0:
+		used = append(used, nonlabeled...)
+	}
 	fi.Body = &hclsyntax.Body{
 		Blocks:     used,
 		Attributes: fi.Body.(*hclsyntax.Body).Attributes,
