@@ -131,11 +131,13 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 	}
 	// If the old -f flag is given convert them to the URL format. If both are given,
 	// cobra would throw an error since they are marked as mutually exclusive.
-	for _, p := range flags.paths {
-		if !isURL(p) {
-			p = "file://" + p
+	if len(flags.toURLs) == 0 {
+		for _, p := range flags.paths {
+			if !isURL(p) {
+				p = "file://" + p
+			}
+			flags.toURLs = append(flags.toURLs, p)
 		}
-		flags.toURLs = append(flags.toURLs, p)
 	}
 	var (
 		err    error
@@ -179,7 +181,7 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 		return err
 	}
 	defer to.Close()
-	diff, err := computeDiff(ctx, client, from, to, env.DiffOptions()...)
+	diff, err := computeDiff(ctx, client, from, to, diffOptions(cmd, env)...)
 	if err != nil {
 		return err
 	}
@@ -431,17 +433,13 @@ func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags, env *E
 			return fmt.Errorf("parse log format: %w", err)
 		}
 	}
-	diff, err := computeDiff(ctx, c, from, to, env.DiffOptions()...)
+	diff, err := computeDiff(ctx, c, from, to, diffOptions(cmd, env)...)
 	if err != nil {
 		return err
 	}
-	return format.Execute(cmd.OutOrStdout(), &cmdlog.SchemaDiff{
-		Client:  c,
-		Context: ctx,
-		From:    diff.from,
-		To:      diff.to,
-		Changes: diff.changes,
-	})
+	return format.Execute(cmd.OutOrStdout(),
+		cmdlog.NewSchemaDiff(ctx, c, diff.from, diff.to, diff.changes),
+	)
 }
 
 type schemaInspectFlags struct {
@@ -545,11 +543,7 @@ func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags,
 	if err != nil {
 		return err
 	}
-	return format.Execute(cmd.OutOrStdout(), &cmdlog.SchemaInspect{
-		Client:  client,
-		Context: ctx,
-		Realm:   s,
-	})
+	return format.Execute(cmd.OutOrStdout(), cmdlog.NewSchemaInspect(ctx, client, s))
 }
 
 // schemaFmtCmd represents the 'atlas schema fmt' subcommand.
@@ -607,7 +601,7 @@ func selectEnv(cmd *cobra.Command) (*Env, error) {
 		if len(envs) != 0 {
 			return nil, fmt.Errorf("unexpected number of envs found: %d", len(envs))
 		}
-		return &Env{Lint: p.Lint, Diff: p.Diff, cfg: p.cfg, Migration: &Migration{}}, nil
+		return &Env{Lint: p.Lint, Diff: p.Diff, Migration: &Migration{}, config: p}, nil
 	// No config nor env was passed.
 	case name == "":
 		return &Env{Lint: &Lint{}, Migration: &Migration{}}, nil
@@ -661,6 +655,10 @@ func setSchemaEnvFlags(cmd *cobra.Command, env *Env) error {
 		return err
 	}
 	switch cmd.Name() {
+	case "inspect":
+		if err := maySetFlag(cmd, flagFormat, env.Format.Schema.Inspect); err != nil {
+			return err
+		}
 	case "apply":
 		if err := maySetFlag(cmd, flagFormat, env.Format.Schema.Apply); err != nil {
 			return err
