@@ -2,8 +2,6 @@
 // This source code is licensed under the Apache 2.0 license found
 // in the LICENSE file in the root directory of this source tree.
 
-//go:build !windows
-
 package pgparse
 
 import (
@@ -172,6 +170,10 @@ func (p *Parser) FixChange(_ migrate.Driver, s string, changes schema.Changes) (
 					drop.Extra = append(drop.Extra, &postgres.Concurrently{})
 				}
 			}
+		case stmt.GetAlterTableStmt() != nil:
+			if fixed, err := FixAlterTable(s, stmt.GetAlterTableStmt(), changes); err == nil {
+				changes = fixed // Make ALTER fixes optional.
+			}
 		}
 	}
 	return changes, nil
