@@ -52,9 +52,9 @@ func main() {
 	update := checkForUpdate(ctx)
 	err := cmdapi.Root.ExecuteContext(ctx)
 	if u := update(); u != "" {
-		fmt.Println(u)
+		fmt.Fprintln(os.Stderr, u)
 	}
-	done()
+	done(err)
 	if err != nil {
 		os.Exit(1)
 	}
