@@ -11,7 +11,7 @@ module.exports = {
   themeConfig: {
     image: "https://og.atlasgo.io/image?title=Schema%20Migration%20Tool%20for%20any%20Language",
     prism: {
-      additionalLanguages: ['hcl', 'java', 'properties', 'gradle'],
+      additionalLanguages: ['hcl', 'java', 'properties', 'gradle', 'php'],
       magicComments: [
         {
           className: 'theme-code-block-highlighted-line',
@@ -54,7 +54,7 @@ module.exports = {
           className: 'header-discord-link',
           position: 'right',
         },{
-          href: 'https://twitter.com/ariga_io',
+          href: 'https://twitter.com/atlasgo_io',
           className: 'header-twitter-link',
           position: 'right',
         },{
@@ -96,7 +96,7 @@ module.exports = {
           "title": "Docs",
           "items": [
             {"label": "Getting Started", "to": "getting-started"},
-            {"label": "Data Definition Language ", "to": "guides/ddl"},
+            {"label": "Atlas vs Others ", "to": "atlas-vs-others"},
             {"label": "CLI Reference", "to": "cli-reference"},
             {"label": "Blog", "to": "blog"},
             {"label": "Guides", "to": "guides"},
@@ -117,6 +117,7 @@ module.exports = {
           "title": "Integrations",
           "items": [
             {"label": "GitHub Actions", "to": "/integrations/github-actions"},
+            {"label": "Kubernetes Operator", "to": "/integrations/kubernetes/operator"},
             {"label": "Terraform", "to": "/integrations/terraform-provider"},
             {"label": "Go API", "to": "/integrations/go-api"}
           ]
@@ -147,11 +148,9 @@ module.exports = {
       `,
     },
     announcementBar: {
-      id: 'announcementBar-10', // Increment on change
-      content: `<strong>Upcoming Pricing Changes to Atlas Cloud. </strong><a class="cta" href="https://atlasgo.io/blog/2024/03/03/pricing-change">Read all about it here.</a>`,
+      id: 'announcementBar-13', // Increment on change
+      content: `<strong>Atlas v0.23: Redshift, CircleCI, GORM Views, and more. </strong><a class="cta" href="https://atlasgo.io/blog/2024/05/21/atlas-v-0-23">Read it here</a>`,
       isCloseable: true,
-      backgroundColor: '#0084ec',
-      textColor: '#fff',
     },
   },
   plugins: [
@@ -268,6 +267,7 @@ module.exports = {
     ],
     "./plugins/utm-plugin",
     "./plugins/intercom",
+    "./plugins/page-view",
   ],
   presets: [
     [
