@@ -263,6 +263,7 @@ func TestExecutor_Replay(t *testing.T) {
 
 func TestExecutor_Pending(t *testing.T) {
 	var (
+		ctx  = context.Background()
 		drv  = &mockDriver{}
 		rrw  = &mockRevisionReadWriter{}
 		log  = &mockLogger{}
@@ -289,96 +290,229 @@ func TestExecutor_Pending(t *testing.T) {
 			Hash:        "+O40cAXHgvMClnynHd5wggPAeZAk7zSEaNgzXCZOfmY=",
 		}
 	)
-	dir, err := migrate.NewLocalDir(filepath.Join("testdata/migrate", "sub"))
+	dir, err := migrate.NewLocalDir(filepath.Join("testdata", "migrate", "sub"))
 	require.NoError(t, err)
 	ex, err := migrate.NewExecutor(drv, dir, rrw, migrate.WithLogger(log))
 	require.NoError(t, err)
 
 	// All are pending
-	p, err := ex.Pending(context.Background())
+	p, err := ex.Pending(ctx)
 	require.NoError(t, err)
 	require.Len(t, p, 3)
 
 	// 2 are pending.
 	*rrw = []*migrate.Revision{rev1}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.NoError(t, err)
 	require.Len(t, p, 2)
 
 	// Only the last one is pending (in full).
 	*rrw = []*migrate.Revision{rev1, rev2}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.NoError(t, err)
 	require.Len(t, p, 1)
 
 	// First statement of last one is marked as applied, second isn't. Third file is still pending.
 	*rrw = []*migrate.Revision{rev1, rev2, rev3}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.NoError(t, err)
 	require.Len(t, p, 1)
 
 	// Nothing to do if all migrations are applied.
 	rev3.Applied = rev3.Total
 	*rrw = []*migrate.Revision{rev1, rev2, rev3}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
 	require.Len(t, p, 0)
 
 	// If there is a revision in the past with no existing migration, we don't care.
 	rev3.Applied = rev3.Total
 	*rrw = []*migrate.Revision{{Version: "2.11"}, rev3}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
 	require.Len(t, p, 0)
 
 	// If only the last migration file is applied, we expect there are no pending files.
 	*rrw = []*migrate.Revision{rev3}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
 	require.Len(t, p, 0)
 
 	// If the last applied revision has no matching migration file, and the last
 	// migration version precedes that revision, there is nothing to do.
 	*rrw = []*migrate.Revision{{Version: "5"}}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
 	require.Len(t, p, 0)
 
 	// The applied revision precedes every migration file. Expect all files pending.
 	*rrw = []*migrate.Revision{{Version: "1.1"}}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.NoError(t, err)
 	require.Len(t, p, 3)
 
 	// All except one file are applied. The latest revision does not exist and its version is smaller than the last
 	// migration file. Expect the last file pending.
 	*rrw = []*migrate.Revision{rev1, rev2, {Version: "2.11"}}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.NoError(t, err)
 	require.Len(t, p, 1)
 	require.Equal(t, rev3.Version, p[0].Version())
 
 	// If the last revision is partially applied and the matching migration file does not exist, we have a problem.
 	*rrw = []*migrate.Revision{{Version: "deleted", Description: "desc", Total: 1}}
-	p, err = ex.Pending(context.Background())
+	p, err = ex.Pending(ctx)
 	require.EqualError(t, err, migrate.MissingMigrationError{Version: "deleted", Description: "desc"}.Error())
 	require.Len(t, p, 0)
+
+	// If the last revision is a partially applied checkpoint, expect it to be part of
+	// the pending files. Checkpoint files following the failed one are not included.
+	dir, err = migrate.NewLocalDir(filepath.Join("testdata", "partial-checkpoint"))
+	require.NoError(t, err)
+	ex, err = migrate.NewExecutor(drv, dir, rrw, migrate.WithLogger(log))
+	require.NoError(t, err)
+	*rrw = []*migrate.Revision{{Version: "3", Description: "checkpoint", Total: 2, Applied: 1}}
+	p, err = ex.Pending(ctx)
+	require.NoError(t, err)
+	require.Len(t, p, 3)
+	require.Equal(t, "3", p[0].Version())
+	require.Equal(t, "4", p[1].Version())
+	require.Equal(t, "6", p[2].Version())
+}
+
+func TestExecutor_ExecOrderLinear(t *testing.T) {
+	var (
+		drv = &mockDriver{}
+		ctx = context.Background()
+		rrw = &mockRevisionReadWriter{{Version: "1"}, {Version: "2"}, {Version: "3"}}
+		dir = func(names ...string) migrate.Dir {
+			m := &migrate.MemDir{}
+			for _, n := range names {
+				require.NoError(t, m.WriteFile(n, nil))
+			}
+			h, err := m.Checksum()
+			require.NoError(t, err)
+			require.NoError(t, migrate.WriteSumFile(m, h))
+			return m
+		}
+	)
+	t.Run("Linear", func(t *testing.T) {
+		ex, err := migrate.NewExecutor(drv, dir(), rrw)
+		require.NoError(t, err)
+		files, err := ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "3.sql"), rrw)
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "2.5.sql", "3.sql"), rrw)
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorAs(t, err, new(*migrate.HistoryNonLinearError))
+		require.EqualError(t, err, "migration file 2.5.sql was added out of order. See: https://atlasgo.io/versioned/apply#non-linear-error")
+
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "2.5.sql", "2.6.sql", "3.sql"), rrw)
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorAs(t, err, new(*migrate.HistoryNonLinearError))
+		require.EqualError(t, err, "migration files 2.5.sql, 2.6.sql were added out of order. See: https://atlasgo.io/versioned/apply#non-linear-error")
+
+		// The first file executed as checkpoint, therefore, 1.sql is not pending nor skipped.
+		rrw = &mockRevisionReadWriter{{Version: "2"}, {Version: "3"}}
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2_checkpoint.sql", "3.sql"), rrw)
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		// The first file executed as checkpoint, therefore, 1.sql is not pending nor skipped.
+		rrw = &mockRevisionReadWriter{{Version: "2"}, {Version: "3"}}
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2_checkpoint.sql", "2.5.sql", "3.sql"), rrw)
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorAs(t, err, new(*migrate.HistoryNonLinearError))
+		require.EqualError(t, err, "migration file 2.5.sql was added out of order. See: https://atlasgo.io/versioned/apply#non-linear-error")
+	})
+
+	t.Run("LinearSkipped", func(t *testing.T) {
+		ex, err := migrate.NewExecutor(drv, dir(), rrw, migrate.WithExecOrder(migrate.ExecOrderLinearSkip))
+		require.NoError(t, err)
+		files, err := ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "3.sql"), rrw, migrate.WithExecOrder(migrate.ExecOrderLinearSkip))
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		// File 2.5.sql is skipped.
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "2.5.sql", "3.sql"), rrw, migrate.WithExecOrder(migrate.ExecOrderLinearSkip))
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		// Files 2.5.sql and 2.6.sql are skipped.
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "2.5.sql", "2.6.sql", "3.sql"), rrw, migrate.WithExecOrder(migrate.ExecOrderLinearSkip))
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+	})
+
+	t.Run("NonLinear", func(t *testing.T) {
+		ex, err := migrate.NewExecutor(drv, dir(), rrw, migrate.WithExecOrder(migrate.ExecOrderNonLinear))
+		require.NoError(t, err)
+		files, err := ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "3.sql"), rrw, migrate.WithExecOrder(migrate.ExecOrderNonLinear))
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+		require.Empty(t, files)
+
+		// File 2.5.sql is pending.
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "2.5.sql", "3.sql"), rrw, migrate.WithExecOrder(migrate.ExecOrderNonLinear))
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.NoError(t, err)
+		require.Len(t, files, 1)
+		require.Equal(t, "2.5.sql", files[0].Name())
+
+		// Files 2.5.sql, 2.6.sql and 4.sql are pending.
+		ex, err = migrate.NewExecutor(drv, dir("1.sql", "2.sql", "2.5.sql", "2.6.sql", "3.sql", "4.sql"), rrw, migrate.WithExecOrder(migrate.ExecOrderNonLinear))
+		require.NoError(t, err)
+		files, err = ex.Pending(ctx)
+		require.NoError(t, err)
+		require.Len(t, files, 3)
+		require.Equal(t, "2.5.sql", files[0].Name())
+		require.Equal(t, "2.6.sql", files[1].Name())
+		require.Equal(t, "4.sql", files[2].Name())
+	})
 }
 
 func TestExecutor(t *testing.T) {
 	// Passing nil raises error.
 	ex, err := migrate.NewExecutor(nil, nil, nil)
-	require.EqualError(t, err, "sql/migrate: execute: no driver given")
+	require.EqualError(t, err, "sql/migrate: no driver given")
 	require.Nil(t, ex)
 
 	ex, err = migrate.NewExecutor(&mockDriver{}, nil, nil)
-	require.EqualError(t, err, "sql/migrate: execute: no dir given")
+	require.EqualError(t, err, "sql/migrate: no dir given")
 	require.Nil(t, ex)
 
 	dir, err := migrate.NewLocalDir(t.TempDir())
 	require.NoError(t, err)
 	ex, err = migrate.NewExecutor(&mockDriver{}, dir, nil)
-	require.EqualError(t, err, "sql/migrate: execute: no revision storage given")
+	require.EqualError(t, err, "sql/migrate: no revision storage given")
 	require.Nil(t, ex)
 
 	// Does not operate on invalid migration dir.
@@ -514,7 +648,7 @@ func TestExecutor(t *testing.T) {
 	// Test ExecuteTo.
 	*rrw = []*migrate.Revision{}
 	*drv = mockDriver{}
-	require.EqualError(t, ex.ExecuteTo(context.Background(), ""), "sql/migrate: execute: migration with version \"\" not found")
+	require.EqualError(t, ex.ExecuteTo(context.Background(), ""), "sql/migrate: migration with version \"\" not found")
 	require.NoError(t, ex.ExecuteTo(context.Background(), "2.10.x-20"))
 	requireEqualRevisions(t, []*migrate.Revision{rev1, rev2}, *rrw)
 }
