@@ -29,7 +29,12 @@ type inspect struct{ *conn }
 var _ schema.Inspector = (*inspect)(nil)
 
 // InspectRealm returns schema descriptions of all resources in the given realm.
-func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOption) (*schema.Realm, error) {
+func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOption) (_ *schema.Realm, rerr error) {
+	undo, err := i.noSearchPath(ctx)
+	if err != nil {
+		return nil, err
+	}
+	defer func() { rerr = errors.Join(rerr, undo()) }()
 	schemas, err := i.schemas(ctx, opts)
 	if err != nil {
 		return nil, err
@@ -70,7 +75,7 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 			if err := i.inspectSequences(ctx, r, nil); err != nil {
 				return nil, err
 			}
-			if err := i.inspectExtensions(ctx, r, nil); err != nil {
+			if err := i.inspectRealmObjects(ctx, r, nil); err != nil {
 				return nil, err
 			}
 		}
@@ -86,6 +91,34 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 	return schema.ExcludeRealm(r, opts.Exclude)
 }
 
+// noSearchPath ensures the session search_path is clean when inspecting realms to ensures all
+// referenced objects in the public schema (or any other default search_path) are returned
+// qualified in the inspection.
+func (i *inspect) noSearchPath(ctx context.Context) (func() error, error) {
+	if i.crdb {
+		// Skip logic for CockroachDB.
+		return func() error { return nil }, nil
+	}
+	rows, err := i.QueryContext(ctx, "SELECT current_setting('search_path'), set_config('search_path', '', false)")
+	if err != nil {
+		return nil, err
+	}
+	var prev sql.NullString
+	if err := sqlx.ScanOne(rows, &prev, &sql.NullString{}); err != nil {
+		return nil, err
+	}
+	return func() error {
+		if sqlx.ValidString(prev) {
+			rows, err := i.QueryContext(ctx, "SELECT set_config('search_path', $1, false)", prev.String)
+			if err != nil {
+				return err
+			}
+			return rows.Close()
+		}
+		return nil
+	}, nil
+}
+
 // InspectSchema returns schema descriptions of the tables in the given schema.
 // If the schema name is empty, the result will be the attached schema.
 func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.InspectOptions) (s *schema.Schema, err error) {
@@ -383,6 +416,8 @@ func (i *inspect) underlyingType(s *schema.Schema, u *UserDefinedType) schema.Ty
 				return e
 			} else if d, ok := o.(*DomainType); ok && d.T == name {
 				return d
+			} else if c, ok := o.(*CompositeType); ok && c.T == name {
+				return c
 			}
 		}
 	}
@@ -498,13 +533,13 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows, scope queryScope)
 	names := make(map[string]*schema.Index)
 	for rows.Next() {
 		var (
-			table, name, typ                                                      string
-			uniq, primary, included, nullsnotdistinct                             bool
-			desc, nullsfirst, nullslast, opcdefault                               sql.NullBool
-			column, constraints, pred, expr, comment, options, opcname, opcparams sql.NullString
+			table, name, typ                                                              string
+			uniq, primary, included, nullsnotdistinct                                     bool
+			desc, nullsfirst, nullslast, opcdefault                                       sql.NullBool
+			column, constraints, pred, expr, comment, options, opcname, opcparams, exoper sql.NullString
 		)
 		if err := rows.Scan(
-			&table, &name, &typ, &column, &included, &primary, &uniq, &constraints, &pred, &expr, &desc,
+			&table, &name, &typ, &column, &included, &primary, &uniq, &exoper, &constraints, &pred, &expr, &desc,
 			&nullsfirst, &nullslast, &comment, &options, &opcname, &opcdefault, &opcparams, &nullsnotdistinct,
 		); err != nil {
 			return fmt.Errorf("postgres: scanning indexes for schema %q: %w", s.Name, err)
@@ -564,6 +599,9 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows, scope queryScope)
 				NullsLast:  nullslast.Bool,
 			})
 		}
+		if sqlx.ValidString(exoper) {
+			part.AddAttrs(NewOperator(i.schema, exoper.String))
+		}
 		switch {
 		case included:
 			c, ok := scope.column(table, column.String)
@@ -687,7 +725,8 @@ func (i *inspect) checks(ctx context.Context, s *schema.Schema) error {
 
 // addChecks scans the rows and adds the checks to the table.
 func (i *inspect) addChecks(s *schema.Schema, rows *sql.Rows) error {
-	names := make(map[string]*schema.Check)
+	type tc struct{ t, n string }
+	names := make(map[tc]*schema.Check)
 	for rows.Next() {
 		var (
 			noInherit                            bool
@@ -700,20 +739,22 @@ func (i *inspect) addChecks(s *schema.Schema, rows *sql.Rows) error {
 		if !ok {
 			return fmt.Errorf("table %q was not found in schema", table)
 		}
-		if _, ok := t.Column(column); !ok {
+		c, ok := t.Column(column)
+		if !ok {
 			return fmt.Errorf("postgres: column %q was not found for check %q", column, name)
 		}
-		check, ok := names[name]
+		ck, ok := names[tc{t: table, n: name}]
 		if !ok {
-			check = &schema.Check{Name: name, Expr: clause, Attrs: []schema.Attr{&CheckColumns{}}}
+			ck = &schema.Check{Name: name, Expr: clause, Attrs: []schema.Attr{&CheckColumns{}}}
 			if noInherit {
-				check.Attrs = append(check.Attrs, &NoInherit{})
+				ck.AddAttrs(&NoInherit{})
 			}
-			names[name] = check
-			t.Attrs = append(t.Attrs, check)
+			names[tc{t: table, n: name}] = ck
+			t.AddAttrs(ck)
 		}
-		c := check.Attrs[0].(*CheckColumns)
-		c.Columns = append(c.Columns, column)
+		c.AddAttrs(ck)
+		attr := ck.Attrs[0].(*CheckColumns)
+		attr.Columns = append(attr.Columns, column)
 	}
 	return nil
 }
@@ -894,6 +935,18 @@ type (
 		Deps    []schema.Object // Objects this domain depends on.
 	}
 
+	// CompositeType defines a composite type.
+	// https://www.postgresql.org/docs/current/rowtypes.html
+	CompositeType struct {
+		schema.Type
+		schema.Object
+		T      string           // Type name.
+		Schema *schema.Schema   // Optional schema.
+		Fields []*schema.Column // Type fields, also known as attributes/columns.
+		Attrs  []schema.Attr    // Extra attributes, such as OID.
+		Deps   []schema.Object  // Objects this domain depends on.
+	}
+
 	// IntervalType defines an interval type.
 	// https://postgresql.org/docs/current/datatype-datetime.html
 	IntervalType struct {
@@ -959,6 +1012,13 @@ type (
 		T string
 	}
 
+	// ConvertUsing describes the USING clause to convert
+	// one type to another.
+	ConvertUsing struct {
+		schema.Clause
+		X string // Conversion expression.
+	}
+
 	// Constraint describes a postgres constraint.
 	// https://postgresql.org/docs/current/catalog-pg-constraint.html
 	Constraint struct {
@@ -967,6 +1027,21 @@ type (
 		T string // c, f, p, u, t, x.
 	}
 
+	// Operator describes an operator.
+	// https://www.postgresql.org/docs/current/sql-createoperator.html
+	Operator struct {
+		schema.Attr
+		schema.Object
+		// Schema where the operator is defined. If nil, the operator
+		// is not managed by the current scope.
+		Schema *schema.Schema
+		// Operator name. Might include the schema name if the schema
+		// is not managed by the current scope or extension based.
+		// e.g., "public.&&".
+		Name  string
+		Attrs []schema.Attr
+	}
+
 	// Sequence defines (the supported) sequence options.
 	// https://postgresql.org/docs/current/sql-createsequence.html
 	Sequence struct {
@@ -1065,6 +1140,12 @@ type (
 		schema.Clause
 	}
 
+	// NotValid describes the NOT VALID clause for the creation
+	// of check and foreign-key constraints.
+	NotValid struct {
+		schema.Clause
+	}
+
 	// NoInherit attribute defines the NO INHERIT flag for CHECK constraint.
 	// https://postgresql.org/docs/current/catalog-pg-constraint.html
 	NoInherit struct {
@@ -1133,6 +1214,16 @@ func (d *DomainType) Underlying() schema.Type {
 	return d.Type
 }
 
+// SpecType returns the type of the composite type.
+func (c *CompositeType) SpecType() string {
+	return "composite"
+}
+
+// SpecName returns the name of the composite type.
+func (c *CompositeType) SpecName() string {
+	return c.T
+}
+
 // Underlying returns the underlying type of the array.
 func (a *ArrayType) Underlying() schema.Type {
 	return a.Type
@@ -1166,9 +1257,35 @@ func (o *ReferenceOption) Scan(v any) error {
 	return nil
 }
 
-// IsUnique reports if the type is unique constraint.
+// IsUnique reports if the type is a unique constraint.
 func (c Constraint) IsUnique() bool { return strings.ToLower(c.T) == "u" }
 
+// IsExclude reports if the type is an exclude constraint.
+func (c Constraint) IsExclude() bool { return strings.ToLower(c.T) == "x" }
+
+// UniqueConstraint returns constraint with type "u".
+func UniqueConstraint(name string) *Constraint {
+	return &Constraint{T: "u", N: name}
+}
+
+// ExcludeConstraint returns constraint with type "x".
+func ExcludeConstraint(name string) *Constraint {
+	return &Constraint{T: "x", N: name}
+}
+
+// NewOperator returns the string representation of the operator.
+func NewOperator(scope string, name string) *Operator {
+	// When scanned from the database, the operator is returned as: "<schema>.<operator>".
+	// The common case is that operators are the default and defined in pg_catalog, or are
+	// installed by extensions.
+	if parts := strings.FieldsFunc(name, func(r rune) bool {
+		return r == '.'
+	}); len(parts) == 2 && (scope == "" || parts[0] == "pg_catalog" || parts[0] == scope) {
+		return &Operator{Name: parts[1]}
+	}
+	return &Operator{Name: name}
+}
+
 // IntegerType returns the underlying integer type this serial type represents.
 func (s *SerialType) IntegerType() *schema.IntegerType {
 	t := &schema.IntegerType{T: TypeInteger}
@@ -1402,7 +1519,7 @@ FROM
 	JOIN pg_catalog.pg_namespace AS t2 ON t2.nspname = t1.table_schema
 	JOIN pg_catalog.pg_class AS t3 ON t3.relnamespace = t2.oid AND t3.relname = t1.table_name
 	LEFT JOIN pg_catalog.pg_partitioned_table AS t4 ON t4.partrelid = t3.oid
-	LEFT JOIN pg_depend AS t5 ON t5.objid = t3.oid AND t5.deptype = 'e'
+	LEFT JOIN pg_depend AS t5 ON t5.classid = 'pg_catalog.pg_class'::regclass::oid AND t5.objid = t3.oid AND t5.deptype = 'e'
 WHERE
 	t1.table_type = 'BASE TABLE'
 	AND NOT COALESCE(t3.relispartition, false)
@@ -1425,7 +1542,7 @@ FROM
 	JOIN pg_catalog.pg_namespace AS t2 ON t2.nspname = t1.table_schema
 	JOIN pg_catalog.pg_class AS t3 ON t3.relnamespace = t2.oid AND t3.relname = t1.table_name
 	LEFT JOIN pg_catalog.pg_partitioned_table AS t4 ON t4.partrelid = t3.oid
-	LEFT JOIN pg_depend AS t5 ON t5.objid = t3.oid AND t5.deptype = 'e'
+	LEFT JOIN pg_depend AS t5 ON t5.classid = 'pg_catalog.pg_class'::regclass::oid AND t5.objid = t3.oid AND t5.deptype = 'e'
 WHERE
 	t1.table_type = 'BASE TABLE'
 	AND NOT COALESCE(t3.relispartition, false)
@@ -1570,6 +1687,7 @@ SELECT
 	%s AS included,
 	idx.indisprimary AS primary,
 	idx.indisunique AS unique,
+	(CASE WHEN idx.indisexclusion THEN (SELECT conexclop[idx.ord]::regoper FROM pg_constraint WHERE conindid = idx.indexrelid) END) AS excoper,
 	con.nametypes AS constraints,
 	pg_get_expr(idx.indpred, idx.indrelid) AS predicate,
 	pg_get_indexdef(idx.indexrelid, idx.ord, false) AS expression,
