@@ -23,17 +23,20 @@ import (
 
 type (
 	doc struct {
-		Tables       []*sqlspec.Table    `spec:"table"`
-		Views        []*sqlspec.View     `spec:"view"`
-		Materialized []*sqlspec.View     `spec:"materialized"`
-		Enums        []*enum             `spec:"enum"`
-		Domains      []*domain           `spec:"domain"`
-		Sequences    []*sqlspec.Sequence `spec:"sequence"`
-		Funcs        []*sqlspec.Func     `spec:"function"`
-		Procs        []*sqlspec.Func     `spec:"procedure"`
-		Triggers     []*sqlspec.Trigger  `spec:"trigger"`
-		Extensions   []*extension        `spec:"extension"`
-		Schemas      []*sqlspec.Schema   `spec:"schema"`
+		Tables        []*sqlspec.Table    `spec:"table"`
+		Views         []*sqlspec.View     `spec:"view"`
+		Materialized  []*sqlspec.View     `spec:"materialized"`
+		Enums         []*enum             `spec:"enum"`
+		Domains       []*domain           `spec:"domain"`
+		Composites    []*composite        `spec:"composite"`
+		Sequences     []*sqlspec.Sequence `spec:"sequence"`
+		Funcs         []*sqlspec.Func     `spec:"function"`
+		Procs         []*sqlspec.Func     `spec:"procedure"`
+		Aggregates    []*aggregate        `spec:"aggregate"`
+		Triggers      []*sqlspec.Trigger  `spec:"trigger"`
+		EventTriggers []*eventTrigger     `spec:"event_trigger"`
+		Extensions    []*extension        `spec:"extension"`
+		Schemas       []*sqlspec.Schema   `spec:"schema"`
 	}
 
 	// Enum holds a specification for an enum type.
@@ -45,7 +48,7 @@ type (
 		schemahcl.DefaultExtension
 	}
 
-	// Domain holds a specification for a domain type.
+	// domain holds a specification for a domain type.
 	domain struct {
 		Name      string           `spec:",name"`
 		Qualifier string           `spec:",qualifier"`
@@ -57,6 +60,23 @@ type (
 		schemahcl.DefaultExtension
 	}
 
+	// composite holds a specification for a composite type.
+	composite struct {
+		Name      string            `spec:",name"`
+		Qualifier string            `spec:",qualifier"`
+		Schema    *schemahcl.Ref    `spec:"schema"`
+		Fields    []*compositeField `spec:"field"`
+		schemahcl.DefaultExtension
+	}
+
+	// compositeField holds a specification for a field in a composite type.
+	// The extension might hold optional attributes such as collation.
+	compositeField struct {
+		Name string          `spec:",name"`
+		Type *schemahcl.Type `spec:"type"`
+		schemahcl.DefaultExtension
+	}
+
 	// extension holds a specification for a postgres extension.
 	// Note, extension names are unique within a realm (database).
 	extension struct {
@@ -65,6 +85,26 @@ type (
 		// added to the extension definition.
 		schemahcl.DefaultExtension
 	}
+
+	// eventTrigger holds a specification for a postgres event trigger.
+	// Note, event trigger names are unique within a realm (database).
+	eventTrigger struct {
+		Name string `spec:",name"`
+		// Schema, version and comment are conditionally
+		// added to the extension definition.
+		schemahcl.DefaultExtension
+	}
+
+	// aggregate holds the specification for an aggregation function.
+	aggregate struct {
+		Name      string             `spec:",name"`
+		Qualifier string             `spec:",qualifier"`
+		Schema    *schemahcl.Ref     `spec:"schema"`
+		Args      []*sqlspec.FuncArg `spec:"arg"`
+		// state_type, state_func and rest of the attributes
+		// are appended after the function arguments.
+		schemahcl.DefaultExtension
+	}
 )
 
 // merge merges the doc d1 into d.
@@ -75,10 +115,13 @@ func (d *doc) merge(d1 *doc) {
 	d.Views = append(d.Views, d1.Views...)
 	d.Tables = append(d.Tables, d1.Tables...)
 	d.Domains = append(d.Domains, d1.Domains...)
+	d.Composites = append(d.Composites, d1.Composites...)
 	d.Schemas = append(d.Schemas, d1.Schemas...)
+	d.Aggregates = append(d.Aggregates, d1.Aggregates...)
 	d.Sequences = append(d.Sequences, d1.Sequences...)
 	d.Extensions = append(d.Extensions, d1.Extensions...)
 	d.Triggers = append(d.Triggers, d1.Triggers...)
+	d.EventTriggers = append(d.EventTriggers, d1.EventTriggers...)
 	d.Materialized = append(d.Materialized, d1.Materialized...)
 }
 
@@ -118,10 +161,37 @@ func (d *domain) SetQualifier(q string) { d.Qualifier = q }
 // SchemaRef returns the schema reference for the domain.
 func (d *domain) SchemaRef() *schemahcl.Ref { return d.Schema }
 
+// Label returns the defaults label used for the composite resource.
+func (c *composite) Label() string { return c.Name }
+
+// QualifierLabel returns the qualifier label used for the composite resource, if any.
+func (c *composite) QualifierLabel() string { return c.Qualifier }
+
+// SetQualifier sets the qualifier label used for the composite resource.
+func (c *composite) SetQualifier(q string) { c.Qualifier = q }
+
+// SchemaRef returns the schema reference for the composite.
+func (c *composite) SchemaRef() *schemahcl.Ref { return c.Schema }
+
+// Label returns the defaults label used for the aggregate resource.
+func (a *aggregate) Label() string { return a.Name }
+
+// QualifierLabel returns the qualifier label used for the aggregate resource, if any.
+func (a *aggregate) QualifierLabel() string { return a.Qualifier }
+
+// SetQualifier sets the qualifier label used for the aggregate resource.
+func (a *aggregate) SetQualifier(q string) { a.Qualifier = q }
+
+// SchemaRef returns the schema reference for the aggregate.
+func (a *aggregate) SchemaRef() *schemahcl.Ref { return a.Schema }
+
 func init() {
 	schemahcl.Register("enum", &enum{})
 	schemahcl.Register("domain", &domain{})
+	schemahcl.Register("composite", &composite{})
+	schemahcl.Register("aggregate", &aggregate{})
 	schemahcl.Register("extension", &extension{})
+	schemahcl.Register("event_trigger", &eventTrigger{})
 }
 
 // evalSpec evaluates an Atlas DDL document into v using the input.
@@ -138,12 +208,18 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := convertTypes(&d, v); err != nil {
 			return err
 		}
+		if err := convertAggregate(&d, v); err != nil {
+			return err
+		}
 		if err := convertSequences(d.Tables, d.Sequences, v); err != nil {
 			return err
 		}
 		if err := convertExtensions(d.Extensions, v); err != nil {
 			return err
 		}
+		if err := convertEventTriggers(d.EventTriggers, v); err != nil {
+			return err
+		}
 		if err := normalizeRealm(v); err != nil {
 			return err
 		}
@@ -162,6 +238,9 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := convertTypes(&d, r); err != nil {
 			return err
 		}
+		if err := convertAggregate(&d, r); err != nil {
+			return err
+		}
 		if err := convertSequences(d.Tables, d.Sequences, r); err != nil {
 			return err
 		}
@@ -213,12 +292,18 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 		if err := specutil.QualifyObjects(d.Materialized); err != nil {
 			return nil, err
 		}
+		if err := specutil.QualifyObjects(d.Aggregates); err != nil {
+			return nil, err
+		}
 		if err := specutil.QualifyObjects(d.Enums); err != nil {
 			return nil, err
 		}
 		if err := specutil.QualifyObjects(d.Domains); err != nil {
 			return nil, err
 		}
+		if err := specutil.QualifyObjects(d.Composites); err != nil {
+			return nil, err
+		}
 		if err := specutil.QualifyObjects(d.Sequences); err != nil {
 			return nil, err
 		}
@@ -282,6 +367,12 @@ func convertTable(spec *sqlspec.Table, parent *schema.Schema) (*schema.Table, er
 	if err != nil {
 		return nil, err
 	}
+	if err := convertUnique(spec.Extra, t); err != nil {
+		return nil, err
+	}
+	if err := convertExclude(spec.Extra, t); err != nil {
+		return nil, err
+	}
 	if err := convertPartition(spec.Extra, t); err != nil {
 		return nil, err
 	}
@@ -310,6 +401,24 @@ func convertView(spec *sqlspec.View, parent *schema.Schema) (*schema.View, error
 	return v, nil
 }
 
+// convertUnique converts the unique constraints into indexes.
+func convertUnique(spec schemahcl.Resource, t *schema.Table) error {
+	rs := spec.Resources("unique")
+	for _, r := range rs {
+		var sx sqlspec.Index
+		if err := r.As(&sx); err != nil {
+			return fmt.Errorf("parse %s.unique constraint: %w", t.Name, err)
+		}
+		idx, err := convertIndex(&sx, t)
+		if err != nil {
+			return err
+		}
+		idx.SetUnique(true).AddAttrs(UniqueConstraint(sx.Name))
+		t.AddIndexes(idx)
+	}
+	return nil
+}
+
 // convertPartition converts and appends the partition block into the table attributes if exists.
 func convertPartition(spec schemahcl.Resource, table *schema.Table) error {
 	r, ok := spec.Resource("partition")
@@ -560,6 +669,29 @@ func convertPart(spec *sqlspec.IndexPart, part *schema.IndexPart) error {
 		}
 		part.Attrs = append(part.Attrs, &IndexOpClass{Name: name})
 	}
+	// The following attributes are mutually exclusive.
+	if nulls, ok := spec.Attr("nulls_last"); ok {
+		b, err := nulls.Bool()
+		if err != nil {
+			return err
+		}
+		if b {
+			at := sqlx.AttrOr(part.Attrs, &IndexColumnProperty{})
+			at.NullsLast = true
+			schema.ReplaceOrAppend(&part.Attrs, at)
+		}
+	}
+	if nulls, ok := spec.Attr("nulls_first"); ok {
+		b, err := nulls.Bool()
+		if err != nil {
+			return err
+		}
+		if b {
+			at := sqlx.AttrOr(part.Attrs, &IndexColumnProperty{})
+			at.NullsFirst = true
+			schema.ReplaceOrAppend(&part.Attrs, at)
+		}
+	}
 	return nil
 }
 
@@ -605,6 +737,7 @@ func schemaSpec(s *schema.Schema) (*doc, []*schema.Trigger, error) {
 		Schemas:      []*sqlspec.Schema{spec.Schema},
 		Enums:        make([]*enum, 0, len(s.Objects)),
 		Domains:      make([]*domain, 0, len(s.Objects)),
+		Composites:   make([]*composite, 0, len(s.Objects)),
 	}
 	if err := objectSpec(d, spec, s); err != nil {
 		return nil, nil, err
@@ -613,9 +746,9 @@ func schemaSpec(s *schema.Schema) (*doc, []*schema.Trigger, error) {
 }
 
 // tableSpec converts from a concrete Postgres sqlspec.Table to a schema.Table.
-func tableSpec(table *schema.Table) (*sqlspec.Table, error) {
+func tableSpec(t *schema.Table) (*sqlspec.Table, error) {
 	spec, err := specutil.FromTable(
-		table,
+		t,
 		tableColumnSpec,
 		pkSpec,
 		indexSpec,
@@ -625,7 +758,28 @@ func tableSpec(table *schema.Table) (*sqlspec.Table, error) {
 	if err != nil {
 		return nil, err
 	}
-	if p := (Partition{}); sqlx.Has(table.Attrs, &p) {
+	idxs := make([]*sqlspec.Index, 0, len(spec.Indexes))
+	for i, idx1 := range spec.Indexes {
+		if len(t.Indexes) <= i || t.Indexes[i].Name != idx1.Name {
+			return nil, fmt.Errorf("unexpected spec index %q was not found in table %q", idx1.Name, t.Name)
+		}
+		if c, ok := uniqueConst(t.Indexes[i].Attrs); ok && c.N != "" {
+			spec.Extra.Children = append(spec.Extra.Children, &schemahcl.Resource{
+				Type: "unique",
+				Name: c.N,
+				Attrs: append(
+					[]*schemahcl.Attr{schemahcl.RefsAttr("columns", idx1.Columns...)},
+					idx1.Extra.Attrs...,
+				),
+			})
+		} else if ex, ok := excludeConst(t.Indexes[i].Attrs); !ok {
+			idxs = append(idxs, idx1)
+		} else if err := excludeSpec(spec, idx1, t.Indexes[i], ex); err != nil {
+			return nil, err
+		}
+	}
+	spec.Indexes = idxs
+	if p := (Partition{}); sqlx.Has(t.Attrs, &p) {
 		spec.Extra.Children = append(spec.Extra.Children, fromPartition(p))
 	}
 	return spec, nil
@@ -662,7 +816,14 @@ func indexSpec(idx *schema.Index) (*sqlspec.Index, error) {
 	}
 	// Avoid printing the index type if it is the default.
 	if i := (IndexType{}); sqlx.Has(idx.Attrs, &i) && strings.ToUpper(i.T) != IndexTypeBTree {
-		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("type", strings.ToUpper(i.T)))
+		var attr *schemahcl.Attr
+		switch strings.ToUpper(i.T) {
+		case IndexTypeBRIN, IndexTypeHash, IndexTypeGIN, IndexTypeGiST, IndexTypeSPGiST:
+			attr = specutil.VarAttr("type", strings.ToUpper(i.T))
+		default:
+			attr = schemahcl.StringAttr("type", i.T)
+		}
+		spec.Extra.Attrs = append(spec.Extra.Attrs, attr)
 	}
 	if i := (IndexPredicate{}); sqlx.Has(idx.Attrs, &i) && i.P != "" {
 		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("where", strconv.Quote(i.P)))
@@ -689,18 +850,26 @@ func indexPKSpec(idx *schema.Index, attrs []*schemahcl.Attr) []*schemahcl.Attr {
 }
 
 func partAttr(idx *schema.Index, part *schema.IndexPart, spec *sqlspec.IndexPart) error {
-	var op IndexOpClass
-	if !sqlx.Has(part.Attrs, &op) {
-		return nil
+	if op := (IndexOpClass{}); sqlx.Has(part.Attrs, &op) {
+		switch d, err := op.DefaultFor(idx, part); {
+		case err != nil:
+			return err
+		case d:
+		case len(op.Params) > 0:
+			spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.RawAttr("ops", op.String()))
+		case postgresop.HasClass(op.String()):
+			spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("ops", op.String()))
+		default:
+			spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.StringAttr("ops", op.String()))
+		}
 	}
-	switch d, err := op.DefaultFor(idx, part); {
-	case err != nil:
-		return err
-	case d:
-	case len(op.Params) > 0:
-		spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.RawAttr("ops", op.String()))
-	default:
-		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("ops", op.String()))
+	if nulls := (IndexColumnProperty{}); sqlx.Has(part.Attrs, &nulls) {
+		switch {
+		case part.Desc && nulls.NullsLast:
+			spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.BoolAttr("nulls_last", true))
+		case !part.Desc && nulls.NullsFirst:
+			spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.BoolAttr("nulls_first", true))
+		}
 	}
 	return nil
 }
@@ -748,6 +917,12 @@ func columnTypeSpec(t schema.Type) (*sqlspec.Column, error) {
 				IsRef: true,
 				T:     specutil.ObjectRef(o.Schema, o).V},
 		}, nil
+	case *CompositeType:
+		return &sqlspec.Column{
+			Type: &schemahcl.Type{
+				IsRef: true,
+				T:     specutil.ObjectRef(o.Schema, o).V},
+		}, nil
 	case *DomainType:
 		return &sqlspec.Column{
 			Type: &schemahcl.Type{
@@ -774,6 +949,7 @@ var TypeRegistry = schemahcl.NewRegistry(
 		schemahcl.AliasTypeSpec("character_varying", TypeCharVar, schemahcl.WithAttributes(schemahcl.SizeTypeAttr(false))),
 		schemahcl.NewTypeSpec(TypeChar, schemahcl.WithAttributes(schemahcl.SizeTypeAttr(false))),
 		schemahcl.NewTypeSpec(TypeCharacter, schemahcl.WithAttributes(schemahcl.SizeTypeAttr(false))),
+		schemahcl.NewTypeSpec(TypeBPChar),
 		schemahcl.NewTypeSpec(TypeInt2),
 		schemahcl.NewTypeSpec(TypeInt4),
 		schemahcl.NewTypeSpec(TypeInt8),
@@ -841,7 +1017,7 @@ var TypeRegistry = schemahcl.NewRegistry(
 			typeOID, typeRegClass, typeRegCollation, typeRegConfig, typeRegDictionary, typeRegNamespace,
 			typeName, typeRegOper, typeRegOperator, typeRegProc, typeRegProcedure, typeRegRole, typeRegType,
 			typeAny, typeAnyElement, typeAnyArray, typeAnyNonArray, typeAnyEnum, typeInternal, typeRecord,
-			typeTrigger, typeVoid, typeUnknown,
+			typeTrigger, typeEventTrigger, typeVoid, typeUnknown,
 		} {
 			specs = append(specs, schemahcl.NewTypeSpec(t))
 		}
